// .js wrapper based on the BV exampleplugin to interact with PPI analysis plugin
// Date: 06.11.2015
// Last updated: 13.05.2018
// Author: Rick van Hoof, Rainer Goebel

var bv = BrainVoyager; // use short symbol to access BrainVoyager API
var $find = bv.FindChild;
var $setint = bv.SetPluginIntParameter;
var $getint = bv.GetPluginIntParameter;
var $setstring = bv.SetPluginStringParameter;
var $getstring = bv.GetPluginStringParameter;

var o = new Object;
o.dialogFileName   = "ppi_gui.ui";      // specify name of file that will be used by bv to create your UI
o.dialogAccessName = "PluginWidget";    // specify the name you want to use for accessing your UI

o.initDlg = function()
{
	var dlg = this.PluginWidget; // less to write
    dlg.windowTitle = "PPI Plugin"; // use to change dialog title

    // connect UI elements to script functions - you need to know the names of your elements (check your definition)
    $find("rb_StandardPPI").pressed.connect(this, this.selectStandardPPI);
    $find("rb_GenPPI").pressed.connect(this, this.selectGenPPI);
    $find("pb_Start").clicked.connect(this, this.StartPPI);
    $find("pb_Save").clicked.connect(this, this.Save);
    $find("pb_Load").clicked.connect(this, this.Load);
    $find("pb_LoadVTC").clicked.connect(this, this.AddSubject);
    $find("pb_Clear").clicked.connect(this, this.Clear);
    //$find("cb_SaveMDM").toggled.connect(this, this.SaveMDM);
}
o.Save = function()
{
    bv.PrintToLog("Script> Save as.. button clicked");
    $setstring("Command", "Save");
    bv.ExecutePlugin();
}

o.Load = function()
{
    bv.PrintToLog("Script> Load button clicked");
    $setstring("Command", "Load");
    $find("listWidget_min").clear(); $find("listWidget_zero").clear(); $find("listWidget_plus").clear();
    $find("tbrVTCFileList").clear(); $find("tbrVOIFileList").clear();
    $setstring("VTCFileList", $find("tbrVTCFileList").plainText);
    $setstring("VOIFileList", $find("tbrVOIFileList").plainText);
    bv.ExecutePlugin();

    var CondsLW_zero1 = $find("listWidget_zero"); // changed

    // get result from plugin
    var newVTCFileList = $getstring("VTCFileList");
    var newVOIFileList = $getstring("VOIFileList");
    var prt_cond_header = $getstring("prt_cond_header");
    var saveWeights = $getint("saveWeights");

    // update lists
    $find("tbrVTCFileList").plainText = newVTCFileList;
    $find("tbrVOIFileList").plainText = newVOIFileList;

    //if(($find("rb_StandardPPI").checked) & (saveWeights == 0)){
    if($find("rb_StandardPPI").checked){
        var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
        for(var c=0; c<NrOfConditions; c++)
        {
            var CondNameID = "CondName-" + (c+1);
            var CondName = $getstring(CondNameID);
            CondsLW_zero1.addItem(CondName);
        }
    }
}

o.Clear = function()
{
    $setstring("Command", "Clear");
    $find("listWidget_min").clear(); $find("listWidget_zero").clear(); $find("listWidget_plus").clear();
    $find("tbrVTCFileList").clear();  $find("tbrVOIFileList").clear();

    bv.ExecutePlugin();
}

o.AddSubject = function()
{
    var dlg = this.PluginWidget;
    var CondsLW_min1 = $find("listWidget_min");
    var CondsLW_zero = $find("listWidget_zero");
    var CondsLW_plus1 = $find("listWidget_plus");

    bv.PrintToLog("Script> Add Subject button clicked");
    $setstring("Command", "AddSubject");
    $setstring("VTCFileList", $find("tbrVTCFileList").plainText);
    $setstring("VOIFileList", $find("tbrVOIFileList").plainText);

    if($find("rb_StandardPPI").checked) {
        $setint("whichapproach", 0); }
    else {
        $setint("whichapproach", 1); }

    if($find("cb_AddMC").checked) {
        $setint("addMC", 1); }
    else {
        $setint("addMC", 0); }

    bv.ExecutePlugin();

    // get result from plugin
    var newVTCFileList = $getstring("VTCFileList");
    var newVOIFileList = $getstring("VOIFileList");
    var prt_cond_header = $getstring("prt_cond_header");
    var saveWeights = $getint("saveWeights");

    // update lists
    $find("tbrVTCFileList").plainText = newVTCFileList;
    $find("tbrVOIFileList").plainText = newVOIFileList;

    if(($find("rb_StandardPPI").checked) & (saveWeights == 0)){
        var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
        for(var c=0; c<NrOfConditions; c++)
        {
            var CondNameID = "CondName-" + (c+1);
            var CondName = $getstring(CondNameID);
            CondsLW_zero.addItem(CondName);
        }
    }
    if (!($find("rb_StandardPPI").checked)) {
        $find("label_1").setEnabled(false); $find("label_2").setEnabled(false); $find("label_3").setEnabled(false);
        $find("listWidget_min").setEnabled(false); $find("listWidget_zero").setEnabled(false); $find("listWidget_plus").setEnabled(false);
    }
}

o.StartPPI = function()
{
    var dlg = this.PluginWidget;
    var currentDir = bv.CurrentDirectory;
    var doc = bv.ActiveDocument;
    bv.PrintToLog("Script> Start button clicked");
    bv.PrintToLog("Script> Current directory is: " + bv.CurrentDirectory);

	$setstring("Command", "Start");
	$setstring("VTCFileList", $find("tbrVTCFileList").plainText);
	$setstring("VOIFileList", $find("tbrVOIFileList").plainText);

    // check which approach we want to use
    if($find("rb_StandardPPI").checked) {
        $setint("whichapproach", 0);
        this.ReadWeights();
    }
    else {$setint("whichapproach", 1);}

    // enables adding motion correction parameters
    if($find("cb_AddMC").checked) {
        $setint("addMoCo", 1);
        if($find("cb_zMC").checked){ $setint("zMC", 1);}
        else {$setint("zMC", 0);}

    }
    else {$setint("addMoCo", 0);}

    // enables saving created design matrices in a MDM file
    if($find("cb_SaveMDM").checked) {
        $setint("saveMDM", 1);
        doc.ClearMultiStudyGLMDefinition();
    }
    else {
        $find("cb_PredName").checked = false;
        $setint("saveMDM", 0);
    }
    if($find("cb_PredName").checked){ $setint("changePredName", 1);}
    else {$setint("changePredName", 0);}

    if($find("cb_Baseline").checked){ $setint("baseline", 1);}
    else {$setint("baseline", 0);}

    // run analysis
    bv.ExecutePlugin();

    if($find("cb_SaveMDM").checked) {
        var SDMFileList = $getstring("SDMFileList");
        var VTCFileList = $getstring("VTCFileList");
        var VTCFileNameList = $getstring("VTCFileNameList");
        SDMFileList = SDMFileList.match(/[^\s]+/g); // get substrings
        VTCFileList = VTCFileList.match(/[^\s]+/g);
        VTCFileNameList = VTCFileNameList.match(/[^\s]+/g);
        var nr_subs = $getint("nr_subs");
        for(var c=0; c<nr_subs; c++)
        {
            doc.AddStudyAndDesignMatrix(VTCFileNameList[c], SDMFileList[c]);
        }
        var mdmFileName = $getstring("mdmFileName");
        doc.SaveMultiStudyGLMDefinitionFile(mdmFileName);
        bv.PrintToLog("Script> MDM file has been saved as: " + mdmFileName);
    }
    bv.PrintToLog("Script> Please load the newly created .sdm file(s) or .mdm file in the (multi-study) GLM dialog.");
    dlg.close();
}

o.ReadWeights = function()
{
    bv.PrintToLog("Script> Reading weights");
    var listArray = new Array();
    listArray[0] = $find("listWidget_min");
    listArray[1] = $find("listWidget_zero");
    listArray[2] = $find("listWidget_plus");

    // count number of items in list
    var NCOND = new Array();
    NCOND[0] = listArray[0].count;
    NCOND[1] = listArray[1].count;
    NCOND[2] = listArray[2].count;
    if(NCOND[0] == 0 && NCOND[1] == 0 && NCOND[2] == 0) {
        bv.MessageBox("No protocol conditions available!");
    }
    bv.SetPluginStringParameter("NCon_min", NCOND[0]);
    bv.SetPluginStringParameter("NCon_zero", NCOND[1]);
    bv.SetPluginStringParameter("NCon_plus", NCOND[2]);

    var cidx = 0;
    for (var j=0; j<3; j++) // list loop
    {
        for(var i=0; i<NCOND[j]; i++) // cond loop
        {
            if (NCOND[j] == 0) { break; }
            var CondName = listArray[j].item(i).text();
            var CondIdentifier = cidx + " list" + j + " " +  CondName;
            var CondID = "CondID-" + cidx;
            bv.SetPluginStringParameter(CondID, CondIdentifier);
            cidx++;
        }
    }
}

o.selectStandardPPI = function()
{
    bv.PrintToLog("Script> Standard PPI analysis selected");
    $setstring("Command", "Clear");
    $find("rb_GenPPI").checked = false;
    $find("rb_StandardPPI").checked = true;
    $setint("whichapproach", 0);

    // clear existing list
    $find("listWidget_min").clear(); $find("listWidget_zero").clear(); $find("listWidget_plus").clear();
    $find("tbrVTCFileList").clear(); $find("tbrVOIFileList").clear();

    // dis/enable GUI options
    $find("label_1").setEnabled(true); $find("label_2").setEnabled(true);$find("label_3").setEnabled(true);
    $find("listWidget_min").setEnabled(true); $find("listWidget_zero").setEnabled(true); $find("listWidget_plus").setEnabled(true);
    $find("cb_Baseline").setEnabled(false);

    bv.ExecutePlugin();
}

o.selectGenPPI = function()
{
    bv.PrintToLog("Script> Generalized PPI analysis selected");
    $setstring("Command", "Clear");
    $find("rb_GenPPI").checked = true;
    $find("rb_StandardPPI").checked = false;
    $setint("whichapproach", 1);

    // clear existing list
    $find("listWidget_min").clear(); $find("listWidget_zero").clear(); $find("listWidget_plus").clear();
    $find("tbrVTCFileList").clear(); $find("tbrVOIFileList").clear();

    // disable GUI options
	$find("label_1").setEnabled(false);	$find("label_2").setEnabled(false);	$find("label_3").setEnabled(false);
    $find("listWidget_min").setEnabled(false); $find("listWidget_zero").setEnabled(false); $find("listWidget_plus").setEnabled(false);
    $find("cb_Baseline").setEnabled(true);
}

// It is important to return script object containing properties and functions to be called from BrainVoyager
//
returnScriptObj = function()
{
	return o;
}
returnScriptObj(); // we call this at the end to return created script object to BrainVoyager
